/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <dawn/dawn.h>
#include "../card.h"
#include "../../util/rand.h"
#include "../../display/spritebatch.h"
#include "../../display/texture.h"
#include "../../display/tileset.h"
#include "../../display/shader.h"
#include "../../file/asset.h"

/**
 * Initializes a Teax Hold'em Game for the first time.
 * 
 * @param game Game to initialize.
 */
void holdemMatchInit(holdemmatch_t *match);

/**
 * Initializes a Texas Hold'em Round for the first time. Does not affect the
 * global game state.
 * 
 * @param game Game's round you want to initialize.
 */
void holdemRoundInit(holdemmatch_t *match);

/**
 * Deals a card to the given player, does all the follow up.
 * 
 * @param game Game who's deck to deal from.
 * @param player Player to deal into.
 */
void holdemDeal(holdemmatch_t *match, holdemplayer_t *player);

/**
 * Deal cards to all players.
 * 
 * @param game Game and players to deal around.
 * @param count Count of cards to deal to each player.
 */
void holdemDealAll(holdemmatch_t *match, uint8_t count);

/**
 * Draw the flop, turn or river
 * 
 * @param game Game to flop/turn/river.
 */
void holdemFlop(holdemmatch_t *match);

/**
 * Takes the given bet from a player
 * 
 * @param game Game to add the bet to.
 * @param player Player to take the bet from.
 * @param amount Amount to try and take.
 * @return The real amount that was taken, chips considered.
 */
uint32_t holdemBet(holdemmatch_t *match, holdemplayer_t *player, uint32_t amount);