/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "holdemrender.h"

void holdemRenderPlayer(float x, float y, float z, float yaw) {
  float w, h;
  w = 1, h = (
    (float)HOLDEM_GAME_STATE.kagamiTileset->divY /
    (float)HOLDEM_GAME_STATE.kagamiTileset->divX
  );
  
  int i = (int32_t)(TIME_STATE.current*10)%HOLDEM_GAME_STATE.kagamiTileset->count;
  quadBuffer(HOLDEM_GAME_STATE.kagamiQuad, 0,
    -w/2, -h/2,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].x0,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].y1,
    w/2, h/2,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].x1,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].y0,
    0, 0
  );

  shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.kagamiTexture);
  shaderUsePosition(GAME_STATE.shaderWorld, x, y, z, 0, mathDeg2Rad(yaw), 0);
  primitiveDraw(HOLDEM_GAME_STATE.kagamiQuad, 0, -1);
}

void holdemRenderCard(card_t card,
  float x, float y, float z, float pitch, float yaw, float roll
) {
  tilesetdiv_t *cardFront = HOLDEM_GAME_STATE.cardTileset->divisions + card;
  quadBuffer(HOLDEM_GAME_STATE.cardPrimitive, HOLDEM_GAME_CARD_DEPTH,
    -HOLDEM_GAME_CARD_WIDTH, -HOLDEM_GAME_CARD_HEIGHT, 
    cardFront->x0, cardFront->y1,
    HOLDEM_GAME_CARD_WIDTH, HOLDEM_GAME_CARD_HEIGHT,
    cardFront->x1, cardFront->y0,
    0, 0
  );

  shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.cardTexture);
  shaderUsePosition(GAME_STATE.shaderWorld, x, y, z, pitch, yaw, roll);
  primitiveDraw(HOLDEM_GAME_STATE.cardPrimitive, 0, -1);
}

void holdemRenderWorld() {
  holdemRenderCard(CARD_HEARTS_QUEEN, 0, 0, 0, mathDeg2Rad(-90), 0, 0);

  // Draw the players
  holdemRenderPlayer(0, 0, -1, 0);
  holdemRenderPlayer(-1, 0, -0, 90);
  holdemRenderPlayer(-0.75, 0, 0.75, -45);
  holdemRenderPlayer(0.75, 0, 0.75, 45);
  holdemRenderPlayer(1, 0, 0, -90);
}